using Ewide.Application.Entity;
using Ewide.Application.Service.HouseProjectInfo.Dto;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ewide.Application.Service.HouseProjectInfo
{
    public interface IHouseProjectInfoService
    {
        Task AddProject(AddProjectInput input);
        Task DeleteProject(DeleteProjectInput input);
        Task UpdateProject(UpdateProjectInput input);
        Task<BsHouseProjectInfo> GetById([FromRoute] string projectId);
        Task<dynamic> GetProject([FromQuery] QueryProjectInput input);
        Task<dynamic> QueryProjectPageList([FromQuery] PageProjectInput input);

        Task<int> GetNextProjectSortByAreaCode([FromQuery] ListHouseProjectInfoInput input);
        Task<dynamic> GetProjectList([FromQuery] ListHouseProjectInfoInput input);
    }
}
